; ***********************************************************
; ColorPaleta: establece un color de la paleta
; D: pen number
; E: color number
; Modifica: BC
; ***********************************************************
ColorPaleta:
      ld BC, #7F00   ; Pen selection en gate array
      out (C), D     ; #7F00 + pen number
      out (C), E     ; #7F40 + color number
      ret

; ***********************************************************
; ColoresPaleta: establece una paleta completa
; HL: array de X colores
; B: numero de colores en el array
; Modifica: BC, HL, DE
; ***********************************************************
ColoresPaleta:
      ld D, 0        ; Indice del color
ColoresPaleta_Loop:
      ld E, (HL)
      push BC
      call ColorPaleta 
      pop BC
      inc D
      inc HL
      djnz ColoresPaleta_Loop
      ret

  
; ***********************************************************
; ModoPantalla: establece el modo grafico
; A = modo (0, 1 o 2)
; Modifica: BC
; ***********************************************************
ModoPantalla:
      ld BC, #7F8C   ; Upper#Lower ROM disabled
      add A, C
      out (C), A  ; #7F8C + modo pantalla
      ret

; ***********************************************************
; LimpiaPantalla: borra pantalla con color A
; A: byte a usar
; Modifica: HL, DE, BC
; ***********************************************************
LimpiaPantalla:
      ld HL, MEM_VIDEO
      ld E, L
      ld D, H
      inc DE         ; DE = HL + 1 (#C001)
      ld (HL), A     ; Byte a volcar en pantalla
      ld BC, MEM_VIDEO_TAM - 1 ; 16Kb - 1 (porque DE va uno por delante)
      ldir           ; Copia de HL (que siempre es 0) a DE (que es HL+1)
      ret            ; e incrementa HL y DE y decrementa BC

; ***********************************************************
; ColorBorde: establece el color del borde
; A: color elegido
; Modifica: BC
; ***********************************************************
ColorBorde:
      ld BC, #7F10
      out (C), C  ; Seleccion borde en gate array 
      out (C), A  ; Puerto #7F40 + color
      ret

; ***********************************************************
; VSync: espera al comienzo del retrazo vertical
; ***********************************************************
VSync:
      ld B, #F5            ;; PPI port B input
VSync_Bucle:
      in A, (C)            ;; [4] read PPI port B input
                           ;; (bit 0 = "1" if vsync is active,
                           ;;  or bit 0 = "0" if vsync is in-active)
      rra                  ;; [1] put bit 0 into carry flag
      jp NC, VSync_Bucle   ;; [3] if carry not set, loop, otherwise continue
      ret


; ***********************************************************
; PintaBloque16: pinta un bloque de 16 pixels de ancho
; HL: destino en memoria de video
; DE: origen del sprite
; A: altura del bloque (8 para tiles, 16 para sprites)
; Modifica: DE, BC, HL, A
; ***********************************************************
PintaBloque16:
         ex DE, HL   ; HL origen del sprite, DE destino en pantalla
         ldi         ; Copiamos los 16 pixels (8 bytes) de ancho
         ldi
         ldi
         ldi
         ldi
         ldi
         ldi
         ldi                  
         ld BC, #7F8 ; Primer pixel de la siguiente linea (#800 - 8 bytes) 
         ex DE, HL   ; Ahora, destino en memoria de video en HL
         add HL, BC
         jp NC, PintaBloque16_SiguienteLinea
         ld BC, #C050
         add HL, BC
PintaBloque16_SiguienteLinea
         dec A
         jp NZ, PintaBloque16
         ret   


; ***********************************************************
; PintaBloque8: pinta un bloque de 8 pixels de ancho
; HL: destino en memoria de video
; DE: origen del sprite
; A: altura del bloque (8 para tiles, 16 para sprites)
; Modifica: DE, BC, HL, A
; ***********************************************************
PintaBloque8:
         ex DE, HL   ; HL origen del sprite, DE destino en pantalla
         ldi         ; Copiamos los 8 pixels (4 bytes) de ancho
         ldi
         ldi
         ldi
         ld BC, #7FC ; Primer pixel de la siguiente linea (#800 - 4 bytes) 
         ex DE, HL   ; Ahora, destino en memoria de video en HL
         add HL, BC
         jp NC, PintaBloque8_SiguienteLinea
         ld BC, #C050
         add HL, BC
PintaBloque8_SiguienteLinea
         dec A
         jp NZ, PintaBloque8
         ret   



; ***********************************************************
; PintaPantalla: pinta una pantalla completa de 10x6 tiles.
; Solo se pintan los tiles libres, ya que los ocupados 
; coinciden con el color del fondo de la pantalla.
; Antes de llamar a esta rutina se colorea el borde y el fondo
; de pantalla con el mismo color.
; DE: origen de la definicion de la pantalla
; ***********************************************************
PintaPantalla:
         ld HL, MEM_VIDEO
         ld C, 6        ; 6 filas por pantalla
         ld B, 10       ; 10 tiles por fila
PintaPantalla_BucleFila:
         ld A, (DE)     ; Carga el valor de la casilla, 0 = libre, 1 = ocupada
         cp 1           ; Ocupada?
         jp Z, PintaPantalla_SaltaTile
         push BC        ; Contador de tiles
         push DE        ; DE a tabla de tiles, HL a memoria de video
         push HL
         ; Obtenemos el tile a pintar de la lista de tiles
         ex DE, HL      ; Guardamos HL con la posicion de la memoria de video
         ld HL, TABLA_TILES   ; HL apunta al inicio de la tabla de tiles
         ld B, 0
         ld C, A        ; En BC el numero del tile a pintar
         add HL, BC
         add HL, BC     ; Sumamos dos veces para movernos por TABLA_TILE (words)
         ld C, (HL)     ; Cargamos en BC la direccion del tile (ojo, little endian)
         inc HL
         ld B, (HL)
         ex DE, HL      ; Restauramos HL (DE <-> HL)
         ld D, B        ; Movemos BC a DE para llamar a PintaBloque16
         ld E, C 
         ld A, 32       ; Pînta el tile (16x32 pixels)
         call PintaBloque16
         pop HL
         pop DE
         pop BC 

PintaPantalla_SaltaTile:
         push DE         ; Siguiente tile en la misma fila (HL+8)
         ld DE, 8
         add HL, DE
         pop DE                                                               
         inc DE         ; Siguiente tile en la definicion de pantalla
         djnz PintaPantalla_BucleFila
         
         ld A, C        ; Cambiamos de fila
         dec A          
         ld BC, #F0     ; Sumamos F0 a HL para pasar a la siguiente fila
         add HL, BC
         ld B, 10       ; Reseteamos el contador de tiles por fila
         ld C, A        ; Hemos completado las seis filas?
         cp 0  
         jp NZ, PintaPantalla_BucleFila
         ret



; ***********************************************************
; PintaCabeza: pinta la cabeza del gato en la posicion 
; (GATO_X_NEXT, GATO_Y_NEXT)
; ***********************************************************
PintaCabeza:
         ld HL, #C000   ; Posicion inicial de pantalla
         ; Calculamos la posicion de memoria de video de la fila 
         ; sumando #140xfila a la posicion inicial 
         ld BC, #140
         ld A, E        ; Coordenada Y
PintaCabeza_CoordenadaY:         
         cp 0
         jp Z, PintaCabeza_CoordenadaX
         add HL, BC     
         dec A
         jr PintaCabeza_CoordenadaY
PintaCabeza_CoordenadaX:  
         ; Calculamos el desplazamiento en la fila sumando 8xcolumna
         ld BC, 8
         ld A, D
PintaCabeza_CoordenadaXBucle:
         cp 0
         jp Z, PintaCabeza_Parpadeo
         add HL, BC
         dec A
         jr PintaCabeza_CoordenadaXBucle
PintaCabeza_Parpadeo:
         ; En HL ya esta calculada la posicion de memoria de video
         ; Hacemos parpadear al gato segun CONTADOR_FRAMES
         ; Gato_00 : CONTADOR_FRAMES < 242
         ; Gato_07 : CONTADOR_FRAMES < 246
         ; Gato_08 : CONTADOR_FRAMES < 250
         ; Gato_07 : CONTADOR_FRAMES < 254
         ld A, (CONTADOR_FRAMES)
         cp 242
         jr C, PintaCabeza_OjosAbiertos
         cp 246
         jr C, PintaCabeza_OjosSemiAbiertos
         cp 250
         jr C, PintaCabeza_OjosCerrados
PintaCabeza_OjosSemiAbiertos:         
         ld DE, Gato_07
         jr PintaCabeza_Fin 
PintaCabeza_OjosCerrados:
         ld DE, Gato_08
         jr PintaCabeza_Fin
PintaCabeza_OjosAbiertos:
         ld DE, Gato_00
         jr PintaCabeza_Fin
PintaCabeza_Fin:
         ld A, 32    ; Altura del sprite de la cabeza
         call PintaBloque16
         ret

; ***********************************************************
; PintaCuerpo: pinta el cuerpo del gato en la posicion 
; (GATO_X, GATO_Y). El sprite utilizado depende de la 
; direccion de movimiento actual y anterior, ya que se usan 
; cuatro sprites distintos para hacer las esquinas segun 
; esta tabla:
;
;  GATO_MUEVE     GATO_MUEVE_NEXT   SPRITE
;  ----------     ---------------   ------
;  DIR_LEFT       DIR_LEFT          Gato_01
;  DIR_RIGHT      DIR_RIGHT         Gato_01

;  DIR_UP         DIR_UP            Gato_02
;  DIR_DOWN       DIR_DOWN          Gato_02
;
;  DIR_LEFT       DIR_DOWN          Gato_03
;  DIR_UP         DIR_RIGHT         Gato_03
;
;  DIR_RIGHT      DIR_DOWN          Gato_04
;  DIR_UP         DIR_LEFT          Gato_04
;
;  DIR_RIGHT      DIR_UP            Gato_05
;  DIR_DOWN       DIR_LEFT          Gato_05
;
;  DIR_LEFT       DIR_UP            Gato_06
;  DIR_DOWN       DIR_RIGHT         Gato_06
; ***********************************************************
PintaCuerpo:
         ld HL, #C000   ; Posicion inicial de pantalla
         ; Calculamos la posicion de memoria de video de la fila 
         ; sumando #140xfila a la posicion inicial 
         ld BC, #140
         ld A, E        ; Coordenada Y
PintaCuerpo_CoordenadaY:         
         cp 0
         jp Z, PintaCuerpo_CoordenadaX
         add HL, BC     
         dec A
         jr PintaCuerpo_CoordenadaY
PintaCuerpo_CoordenadaX:  
         ; Calculamos el desplazamiento en la fila sumando 8xcolumna
         ld BC, 8
         ld A, D
PintaCuerpo_CoordenadaXBucle:
         cp 0
         jp Z, PintaCuerpo_FinBucle
         add HL, BC
         dec A
         jr PintaCuerpo_CoordenadaXBucle
PintaCuerpo_FinBucle:
         ; En HL ya esta calculada la posicion de memoria de video
         ; Necesitamos saber cual de los sprites del cuerpo del gato usar
         ; Primera comprobacion: si GATO_MUEVE = GATO_MUEVE_NEXT
         ; usamos los sprites de cuerpo horizontal o vertical
         ; porque el gato se esta moviendo en linea recta
         ld A, (GATO_MUEVE)
         ld B, A
         ld A, (GATO_MUEVE_NEXT)
         cp B
         jr NZ, PintaCuerpo_Giro
         ; El gato se mueve en linea recta
         cp DIR_LEFT    ; Se mueve hacia la izquierda?
         jr NZ, PintaCuerpo_Derecha
         ld DE, Gato_01  ; Sprite del cuerpo horizontal
         jr PintaCuerpo_Fin
PintaCuerpo_Derecha:
         cp DIR_RIGHT   ; Se mueve hacia la derecha?
         jr NZ, PintaCuerpo_Arriba
         ld DE, Gato_01  ; Sprite del cuerpo horizontal
         jr PintaCuerpo_Fin
PintaCuerpo_Arriba:
         cp DIR_UP      ; Se mueve hacia arriba?
         jr NZ, PintaCuerpo_Abajo
         ld DE, Gato_02  ; Sprite del cuerpo vertical
         jr PintaCuerpo_Fin
PintaCuerpo_Abajo:
         ld DE, Gato_02  ; Sprite del cuerpo vertical
         jr PintaCuerpo_Fin
         ; El gato ha cambiado de direccion. Necesitamos calcular
         ; cual de las cuatro esquinas pintar a partir de la 
         ; direccion anterior y actual (mirar tabla)
PintaCuerpo_Giro:
         ; En A tenemos GATO_MUEVE_NEXT y en B GATO_MUEVE
         ; y sabemos que son distintos. 
         cp DIR_UP
         jr Z, PintaCuerpo_GiroArriba
         cp DIR_DOWN
         jr Z, PintaCuerpo_GiroAbajo
         cp DIR_LEFT
         jr Z, PintaCuerpo_GiroIzquierda
         ; Giro ?->Derecha
         ld A, B        ; A <- GATO_MUEVE
         cp DIR_UP
         jr Z, PintaCuerpo_GiroArribaDerecha
         ; Giro Abajo->Derecha, sprite Gato_06
         ld DE, Gato_06
         jr PintaCuerpo_Fin
PintaCuerpo_GiroArribaDerecha:
         ; Giro Arriba->Derecha, sprite Gato_03
         ld DE, Gato_03
         jr PintaCuerpo_Fin
PintaCuerpo_GiroArriba:
         ; Giro ?->Arriba
         ld A, B        ; A <- GATO_MUEVE
         cp DIR_RIGHT
         jr NZ, PintaCuerpo_GiroIzquierdaArriba
         ; Giro Derecha->Arriba, sprite Gato_05
         ld DE, Gato_05
         jr PintaCuerpo_Fin
PintaCuerpo_GiroIzquierdaArriba
         ; Giro Izquierda->Arriba, sprite Gato_06
         ld DE, Gato_06
         jr PintaCuerpo_Fin
PintaCuerpo_GiroAbajo:
         ; Giro ?->Abajo
         ld A, B        ; A <- GATO_MUEVE
         cp DIR_RIGHT
         jr NZ, PintaCuerpo_GiroIzquierdaAbajo
         ; Giro Derecha->Abajo, sprite Gato_04
         ld DE, Gato_04
         jr PintaCuerpo_Fin
PintaCuerpo_GiroIzquierdaAbajo:
         ; Giro Izquierda->Abajo, sprite Gato_03
         ld DE, Gato_03
         jr PintaCuerpo_Fin
PintaCuerpo_GiroIzquierda:
         ; Giro ?->Izquierda
         ld A, B        ; A <- GATO_MUEVE
         cp DIR_UP
         jr NZ, PintaCuerpo_GiroAbajoIzquierda
         ; Giro Arriba->Izquierda, sprite Gato_04
         ld DE, Gato_04
         jr PintaCuerpo_Fin
PintaCuerpo_GiroAbajoIzquierda
         ; Giro Abajo->Izquierda, sprite Gato_05
         ld DE, Gato_05
PintaCuerpo_Fin:
         ; En DE ya tenemos el sprite a pintar
         ld A, 32    ; Altura del sprite
         call PintaBloque16
         ret
   
; ***********************************************************
; ColorFondo: establece el fondo de pantalla segun el 
; numero de la pantalla actual sobre la lista COLORES_FONDO. 
; A = indice sobre el array COLORES_FONDO
; ***********************************************************
ColorFondo:
         ; Sacamos el color del fondo de la lista COLORES_FONDO
         ; usando los 3 bits menos significativos de PANTALLA_NUM
         and %00000111
         ld HL, COLORES_FONDO
         ld D, 0
         ld E, A     ; Sumamos el indice (PANTALLA_NUM % 8)
         add HL, DE 
         ld D, 8  ; Usamos el pen 8 para el fondo
         ld A, (HL)  ; Color seleccionado en A
         ld E, A
         call ColorPaleta
         ; El color del borde debe coincidir con el fondo de pantalla
         call ColorBorde
         ; Borramos la pantalla con el color de fondo
         ret


; ***********************************************************
; PintaBloque: pinta un bloque de 16x32
; DE: posicion (X,Y)
; BC: direccion del bloque
; ***********************************************************
PintaBloque:
         push BC
         ld HL, #C000   ; Posicion inicial de pantalla
         ; Calculamos la posicion de memoria de video de la fila 
         ; sumando #140xfila a la posicion inicial 
         ld BC, #140
         ld A, E        ; Coordenada Y
PintaBloque_CoordenadaY:         
         cp 0
         jp Z, PintaBloque_CoordenadaX
         add HL, BC     
         dec A
         jr PintaBloque_CoordenadaY
PintaBloque_CoordenadaX:  
         ; Calculamos el desplazamiento en la fila sumando 8xcolumna
         ld BC, 8
         ld A, D
PintaBloque_CoordenadaXBucle:
         cp 0
         jp Z, PintaBloque_Fin
         add HL, BC
         dec A
         jr PintaBloque_CoordenadaXBucle
PintaBloque_Fin:
         ; En HL ya esta calculada la posicion de memoria de video
         pop DE      ; Direccion del bloque (push BC)
         ld A, 32    ; Altura del sprite de la cabeza
         call PintaBloque16
         ret         

; ***********************************************************
; GuardaFondoPantalla: guarda A lineas completas de pixels
; en el buffer BUFFER_PANTALLA o viceversa.
; A: numero de lineas completas
; HL: direccion de memoria de video de origen
; HL tiene que estar alineada a una fila de texto (C000, C050, C0A0, C0F0...)
; ***********************************************************
GuardaFondoPantalla:
      ld DE, BUFFER_PANTALLA
GuardaFondoPantalla_Bucle:
      ; Copia una linea completa de la pantalla
      ld BC, 80    ; 80 bytes
      ldir
      ; Saltamos a la siguiente linea de pantalla
      ld BC, #7B0  ; #800 - 80 bytes del ultimo ldir
      add HL, BC
      jr NC, GuardaFondoPantalla_BucleCont   
      ld BC, #C050 ; Pasamos a la siguiente linea de texto
      add HL, BC    
GuardaFondoPantalla_BucleCont:        
      ; Hemos terminado?
      dec A
      cp 0
      jr NZ, GuardaFondoPantalla_Bucle
      ret

; ***********************************************************
; RestauraFondoPantalla: vuelve a pintar el fondo guardado
; en el buffer BUFFER_PANTALLA.
; A: numero de lineas completas
; HL: direccion de inicio de la memoria de video, alineada 
; a una fila de texto (C000, C050, C0A0, C0F0...)
; ***********************************************************
RestauraFondoPantalla:
      ld DE, BUFFER_PANTALLA
RestauraFondoPantalla_Bucle:
      ex DE, HL   ; HL->BUFFER_PANTALLA, DE->memoria de video
      ; Copia una linea completa de la pantalla
      ld BC, 80    ; 80 bytes
      ldir
      ; Saltamos a la siguiente linea de pantalla
      ld BC, #7B0
      ex DE, HL   ; HL->memoria de video, DE->BUFFER_PANTALLA
      add HL, BC
      jr NC, RestauraFondoPantalla_BucleCont   
      ld BC, #C050 ; Pasamos a la siguiente linea de texto
      add HL, BC    
RestauraFondoPantalla_BucleCont:           
      ; Hemos terminado?
      dec A
      cp 0
      jr NZ, RestauraFondoPantalla_Bucle
      ret 